<?php

namespace Myriad\Authentication\Lib\Guards;

use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Guard;
use Illuminate\Auth\GuardHelpers;
use Illuminate\Auth\TokenGuard;
use Illuminate\Contracts\Auth\UserProvider;
use Illuminate\Http\Request;
use Myriad\Installations\InstallationManager;
use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;

class AccessTokenGuard extends TokenGuard
{
  const AUTH_HEADER = 'Authorization';
  const TOKEN_REGEX_PATTERN = '/[tT]oken ([^\s]+)/';
  const TOKEN_MATCH_INDEX = 1;

  private $installationManager;
  private $accessTokenRepository;

  /**
   * Create a new authentication guard.
   *
   * @param  \Illuminate\Contracts\Auth\UserProvider  $provider
   * @param  \Illuminate\Http\Request  $request
   * @return void
   */
  public function __construct(
    UserProvider $provider,
    Request $request,
    InstallationManager $installationManager,
    IAccessTokenRepository $accessTokenRepository
  ) {
    parent::__construct($provider, $request);

    $this->inputKey = 'token';
    $this->storageKey = 'token';
    $this->installationManager = $installationManager;
    $this->accessTokenRepository = $accessTokenRepository;
  }


  /**
   * Get the token for the current request.
   *
   * @return string
   */
  public function getTokenForRequest()
  {
    $authHeader = $this->request->header(self::AUTH_HEADER);

    $matches = [];

    if (preg_match(self::TOKEN_REGEX_PATTERN, $authHeader, $matches)) {
      return $matches[self::TOKEN_MATCH_INDEX];
    }
  }

  /**
   * Get the currently authenticated user.
   *
   * @return \Illuminate\Contracts\Auth\Authenticatable|null
   */
  public function user()
  {
    // If we've already retrieved the user for the current request we can just
    // return it back immediately. We do not want to fetch the user data on
    // every call to this method because that would be tremendously slow.
    if (! is_null($this->user)) {
      return $this->user;
    }

    $user = null;

    $token = $this->getTokenForRequest();

    if (! empty($token)) {
      $accessToken = $this->accessTokenRepository->getAccessTokenByToken($token);

      if (!empty($accessToken) && $accessToken->client->uuid == $this->installationManager->getUuidFromRequest()) {
        $user = $this->provider->retrieveByCredentials(
          [$this->storageKey => $token]
        );
      }
    }

    return $this->user = $user;
  }
}
