<?php

namespace Myriad\Authentication\Lib\Jobs;

use Illuminate\Contracts\Validation\Factory;
use Illuminate\Validation\ValidationException;
use Myriad\Authentication\Models\User;
use Illuminate\Events\Dispatcher;
use Illuminate\Contracts\Hashing\Hasher;
use Myriad\Authentication\Lib\Events\UserRegisteredWithPhone;
use Myriad\Authentication\Lib\Events\UserRegisteredWithEmail;

class RegisterUser {

  private $request;

  public function __construct(array $request) {
    $this->request = $request;
  }

  public static function getRules() {
    return [
      'first_name' => 'required|string',
      'last_name' => 'required|string',
      'phone' => 'required_without:email|string|unique_verified:user_phones',
      'email' => 'required_without:phone|email|unique_verified:user_emails',
      'type' => 'required|string|in:work,personal',
      'password' => 'required|string|confirmed',
    ];
  }

  public static function validate(Factory $validationFactory, array $request) {
    // validate request
    $validator = $validationFactory->make($request, self::getRules());

    if ($validator->fails()) {
      throw new ValidationException($validator);
    }
  }

  public function handle(
    Factory $validationFactory,
    Dispatcher $eventDispatcher,
    Hasher $hasher
  ) {
    // validate request
    self::validate($validationFactory, $this->request);

    // hash password and create user
    $hashedPassword = $hasher->make($this->request['password']);
    $user = User::create(
      array_merge($this->request, ['password' => $hashedPassword])
    );

    if (is_null($this->request['email'])) {
      $this->addUserPhone($user, $eventDispatcher);
    } else {
      $this->addUserEmail($user, $eventDispatcher);
    }

    return $user;
  }

  private function addUserEmail($user, $dispatcher) {
    // add email address to user
    $userEmail =  $user->emailAddresses()->create([
      'email' => $this->request['email'],
      'type' => $this->request['type'],
      'primary' => true,
      'verified' => false
    ]);

    // fire user email address added event
    $dispatcher->fire(new UserRegisteredWithEmail($user, $userEmail));
  }

  private function addUserPhone($user, $dispatcher) {
    // add phone number to user
    $userPhone =  $user->phoneNumbers()->create([
      'phone' => $this->request['phone'],
      'type' => $this->request['type'],
      'primary' => true,
      'verified' => false
    ]);

    // fire user email address added event
    $dispatcher->fire(new UserRegisteredWithPhone($user, $userPhone));
  }

}

